searchData={"items":[{"type":"module","title":"CSSEx","doc":"CSSEx is a CSS pre-processor inspired by SASS/SCSS. A complete description of what\nit can do can be found in the project's README.md [hex](https://hexdocs.pm/cssex/readme.html) / [github](https://github.com/mnussbaumer/cssex) file.\n\nThis module is the file watcher responsible for automatically processing your CSSEx\nfiles into proper CSS files.\nRefer to the [usage](https://hexdocs.pm/cssex/readme.html#usage) section for details.","ref":"CSSEx.html"},{"type":"function","title":"CSSEx.make_config/2","doc":"Generate a `%CSSEx{}` struct from a keyword list or a map. Its only relevant use case is to \"parse\" app config environment values. You can also pass a directory as the last argument where it will be joined to the paths in the `:entry_points`.\nWhatever the final path it will be expanded when this config is passed as the argument to `start_link/1`","ref":"CSSEx.html#make_config/2"},{"type":"function","title":"CSSEx.start_link/1","doc":"Start a watcher responsible for automatically processing cssex files into css files.\nDefine in the application config something as:\n\n```\nconfig :yourapp_web, CSSEx,\n  entry_points: [\n    {\"../../../../apps/yourapp_web/assets/cssex/app.cssex\", \"../../../../apps/yourapp_web/assets/css/app.css\"}\n]\n```\n\nWith as many `:entry_points` as necessary specified as tuples of `{\"source\", \"dest\"}`\nThen,\n\n```\nApplication.get_env(:yourapp_web, CSSEx)\n|> CSSEx.make_config(Application.app_dir(:your_app_web))\n|> CSSEx.start_link()\n```\n\nOr add it to a supervision tree. Refer to the README.md file.","ref":"CSSEx.html#start_link/1"},{"type":"type","title":"CSSEx.t/0","doc":"","ref":"CSSEx.html#t:t/0"},{"type":"module","title":"CSSEx.HSLA","doc":"Struct and helper functions for generating HSLA values.","ref":"CSSEx.HSLA.html"},{"type":"function","title":"CSSEx.HSLA.from_rgba/1","doc":"Converts an existing `%CSSEx.RGBA{}` struct into a `%CSSEx.HSLA{}` struct.\nTaken from https://www.niwa.nu/2013/05/math-behind-colorspace-conversions-rgb-hsl/","ref":"CSSEx.HSLA.html#from_rgba/1"},{"type":"function","title":"CSSEx.HSLA.new/4","doc":"Generates a `%CSSEx.HSLA{}` wrapped in an :ok tuple, from the values of h, s, l, and alpha. All values are treated as decimal.","ref":"CSSEx.HSLA.html#new/4"},{"type":"function","title":"CSSEx.HSLA.new_hsla/1","doc":"Accepts any value in the form of a binary `\"hsla(0, 10%, 20%, 0.5)\"` or `\"hsl(0, 10%, 20%)\"`, any hexadecimal representation in binary in the form of `\"#xxx\"`, `\"#xxxx\"`, `\"#xxxxxx\"` or `\"#xxxxxxxx\"`, rgb/a as `\"rgba(100,100,100,0.1)\"` or `\"rgb(10,20,30)\"`, or any literal color name defined as web colors (CSSEx.Colors) - returns a `%CSSEx.HSLA{}` struct.","ref":"CSSEx.HSLA.html#new_hsla/1"},{"type":"type","title":"CSSEx.HSLA.t/0","doc":"","ref":"CSSEx.HSLA.html#t:t/0"},{"type":"module","title":"CSSEx.Helpers.Colors","doc":"Web colors taken from https://www.rapidtables.com/web/css/css-color.html","ref":"CSSEx.Helpers.Colors.html"},{"type":"function","title":"CSSEx.Helpers.Colors.colors_tuples/0","doc":"All web colors as list([\"webcolor_identifier\", \"rgb(x,x,x)\"])","ref":"CSSEx.Helpers.Colors.html#colors_tuples/0"},{"type":"module","title":"CSSEx.Helpers.Functions","doc":"Default base functions to use in stylesheets.\n\n```\n@fn::opacity(red, 0.8)\n```","ref":"CSSEx.Helpers.Functions.html"},{"type":"function","title":"CSSEx.Helpers.Functions.darken/3","doc":"Darken function, takes a color in the form of a string and a number representing\nthe percentage to darken and returns a CSS rgba() string.\n\n@fn::darken(orange, 25)","ref":"CSSEx.Helpers.Functions.html#darken/3"},{"type":"function","title":"CSSEx.Helpers.Functions.lighten/3","doc":"Lighten function, takes a color in the form of a string and a number representing\nthe percentage to lighten and returns a CSS rgba() string.\n\n@fn::lighten(orange, 10)","ref":"CSSEx.Helpers.Functions.html#lighten/3"},{"type":"function","title":"CSSEx.Helpers.Functions.opacity/3","doc":"Opacity function, takes a color in the form of a string and a number representing\nthe desired alpha channel value in the form of a float and returns a CSS rgba()\nstring.\n\n@fn::opacity(orange, 0.5)","ref":"CSSEx.Helpers.Functions.html#opacity/3"},{"type":"module","title":"CSSEx.Parser","doc":"The parser module that generates or writes a CSS file based on an entry file.","ref":"CSSEx.Parser.html"},{"type":"function","title":"CSSEx.Parser.parse/1","doc":"Parses a `String.t` or a `charlist` and returns `{:ok, %CSSEx.Parser{}, content_or_empty_list}` or `{:error, %CSSEx.Parser{}}`.\nIf a file path is passed as the final argument it returns the `:ok` tuple with an empty list instead of the content and writes into the file path.\nOn error it returns an :error tuple with the `%CSSEx.Parser{}` having its `:error` key populated.\nIf the first argument is a prefilled `%CSSEx.Parser{}` struct the parser uses that as its basis allowing to provide an `ETS` table that can be retrieved in the end, or passing predefined functions, assigns or variables, prefixes and etc into the context of the parser.","ref":"CSSEx.Parser.html#parse/1"},{"type":"function","title":"CSSEx.Parser.parse/2","doc":"","ref":"CSSEx.Parser.html#parse/2"},{"type":"function","title":"CSSEx.Parser.parse/3","doc":"","ref":"CSSEx.Parser.html#parse/3"},{"type":"function","title":"CSSEx.Parser.parse/4","doc":"","ref":"CSSEx.Parser.html#parse/4"},{"type":"function","title":"CSSEx.Parser.parse_file/2","doc":"Takes a file path to a cssex or css file and parses it into a final CSS\n representation returning either:\n\n```\n{:ok, %CSSEx.Parser{}, final_binary}\n{:error, %CSSEx.Parser{}}\n```\n\nAdditionally a `%CSSEx.Parser{}` struct with prefilled details can be passed as the first\nargument in which case the parser will use it as its configuration. \nYou can also pass a file path as the last argument and instead of returning the final binary on the `:ok` tuple it will write the css directly into that file path and return an empty list instead of the final binary","ref":"CSSEx.Parser.html#parse_file/2"},{"type":"function","title":"CSSEx.Parser.parse_file/3","doc":"","ref":"CSSEx.Parser.html#parse_file/3"},{"type":"function","title":"CSSEx.Parser.parse_file/4","doc":"","ref":"CSSEx.Parser.html#parse_file/4"},{"type":"function","title":"CSSEx.Parser.parse_file/5","doc":"","ref":"CSSEx.Parser.html#parse_file/5"},{"type":"module","title":"CSSEx.RGBA","doc":"Struct and helper functions for generating RGBA values.","ref":"CSSEx.RGBA.html"},{"type":"function","title":"CSSEx.RGBA.from_hsla/1","doc":"Converts an existing `%CSSEx.HSLA{}` struct into a `%CSSEx.RGBA{}` struct.\nTaken from https://www.niwa.nu/2013/05/math-behind-colorspace-conversions-rgb-hsl","ref":"CSSEx.RGBA.html#from_hsla/1"},{"type":"function","title":"CSSEx.RGBA.new/5","doc":"Generates a `%CSSEx.RGBA{}` wrapped in an :ok tuple, from the values of r, g, b, and alpha. All values are treated as decimal by default but another base can be provided as an optional argument.","ref":"CSSEx.RGBA.html#new/5"},{"type":"function","title":"CSSEx.RGBA.new_rgba/1","doc":"Accepts any value in the form of a binary `\"hsla(0, 10%, 20%, 0.5)\"` or `\"hsl(0, 10%, 20%)\"`, any hexadecimal representation in binary in the form of `\"#xxx\"`, `\"#xxxx\"`, `\"#xxxxxx\"` or `\"#xxxxxxxx\"`, rgb/a as `\"rgba(100,100,100,0.1)\"` or `\"rgb(10,20,30)\"`, or any literal color name defined as web colors (CSSEx.Colors) - returns a `%CSSEx.HSLA{}` struct.","ref":"CSSEx.RGBA.html#new_rgba/1"},{"type":"type","title":"CSSEx.RGBA.t/0","doc":"","ref":"CSSEx.RGBA.html#t:t/0"},{"type":"module","title":"CSSEx.Unit","doc":"A basic representation to use for CSS units.","ref":"CSSEx.Unit.html"},{"type":"function","title":"CSSEx.Unit.new_unit/1","doc":"Accepts a String.t and Generates a `%CSSEx.Unit{}` with an appropriate unit set if it matches any of the predefined units, otherwise a struct with unit set to nil when there's no unit, or an error if there's a unit but isn't a valid one.\nIf the argument can't be parsed as a float it returns an `:error` tuple","ref":"CSSEx.Unit.html#new_unit/1"},{"type":"task","title":"mix cssex.parser","doc":"Task to parse cssex files into css files.\nYou can use two types of flags, `--e` (and additionally `--a`) or `--c`.\n\nFor the --e flag you use any number of\n\n`--e path/to_cssex/file.cssex=path/to_css/output.css`\n\nAnd if you want those paths to be relative to some application you can pass it with `--a`\n\n`--a myapp_web`\n\narguments to specify each entry and its output file, or a single path to the cssex where the output file will be in the same directory, with the same file name but the extension cssex replaced by css\n\n`--e path/to_cssex/file.cssex`\n\nThe `--c` flag is used to indicate an entry in the config of the application, in order to read the entry points from there\n\n`--c myapp_web`\n\nSo it would look something like:\n```\nmix cssex.parser --e path/to_cssex/file.cssex=path/to_css/output.css\n```","ref":"Mix.Tasks.Cssex.Parser.html"},{"type":"function","title":"Mix.Tasks.Cssex.Parser.run/1","doc":"Run the parser with `mix cssex.parser`.\n\nRequired arguments:\n\n```\n--e /source/path.cssex=/final/path.css\n--e /source/path.cssex\n--e source/path.cssex=final/path.css --a yourapp_web\n--e source/path.cssex --a yourapp_web\n```\nOr\n```\n--c yourapp_web\n```\n\nWhere `yourapp_web` specifies a config under the key `CSSEx`, with a key of `:entry_points` composed of tuple pairs of source & destination files.","ref":"Mix.Tasks.Cssex.Parser.html#run/1"},{"type":"extras","title":"CSSEx","doc":"# CSSEx\n\nA small pre-processing extension language for CSS written in Elixir.\nIts main purpose is to provide a native Elixir pre-processor for CSS, in the vein of Sass/Scss.\n\n \n      Syntax  &nbsp; | \n      Caveats  &nbsp; | \n      Installation  &nbsp; | \n      Usage  &nbsp; | \n      Tasks  &nbsp; | \n      Dynamic/RunTime parsing  &nbsp; | \n      Internals  &nbsp; | \n      About  &nbsp; | \n      Copyright","ref":"readme.html"},{"type":"extras","title":"Syntax: - CSSEx","doc":"Selectors  \n    Variables  \n    Assigns  \n    @expandable & @apply  \n    Functions  \n    Implemented Functions  \n    EEx Blocks  \n    @at-rules  \n    Comments  \n    Reserved Tokens","ref":"readme.html#syntax"},{"type":"extras","title":"Nested selectors and '&' concatenation - CSSEx","doc":"```css\nbutton {\n    background-color: blue;\n    color: white;\n    padding: 5px;\n    \n    .class_1 {\n      color: yellow;\n        svg&, .child {\n \t  fill: red;\n\t}\n    }\n\n    &.concatenated_class {\n      padding: 10px;\n    }\n\n    @media screen and (max-width: 756px) {\n    \t   font-size: 24px;\n\t   @media and (min-width: 500px) {\n\t   \t  font-weight: 300;\n           }\n    }\n}\n```\n\n##### into\n\n```css\nbutton { background-color: blue; color: white; padding: 5px }\nbutton .class_1  { color: yellow }\nbutton svg.class_1, button .class_1.child { fill: red }\nbutton.concatenated_class { padding: 10px }\n\n@media screen and (max-width: 756px) { button { font-size: 24px } }\n@media screen and (max-width: 756px) and (min-width: 500px) { button { font-weight: 300 } }\n```\n\nThe `&` operator can only be used inside a block and either at the start or end of each selector (or if single inside a `:not(&)`, or `:is(&)`).\nIf it's at the start it will append that selector to the parent, unless it's a tag selector (`p`, `canvas`, `your-custom-element`, etc) in which case it will prepend itself, and raise an error if you're trying to do so with a parent tag.\n\nIf it's at the end then the selector becomes the parent of it's parent.\n\n```css\ndiv, section {\n  &.concat, .parent& {\n    .inner { color: blue; }\n    &.inner-concat { color: red; }\n  }\n}    \n\n```\n\ninto\n\n```css\ndiv.concat .inner,\n.parent div .inner,\nsection.concat .inner,\n.parent section .inner {\n  color:blue\n}\n\ndiv.concat.inner-concat,\n.parent.inner-concat div,\nsection.concat.inner-concat,\n.parent.inner-concat section {\n\n  color:red\n}\n```\n\n\nBare selectors inside blocks create regular selection chains.\n\n`@media` declarations can be nested as well but as of now there's no semantic checks done, which means that if you write something non-sensical it will be placed as is.","ref":"readme.html#nested-selectors-and-concatenation"},{"type":"extras","title":"Variables - CSSEx","doc":"```css\n$!a_variable red;\n$!another_variable 12;\n\ndiv {\n    color: <$a_variable$>;\n    font-size: <$another_variable$>px;\n}\n```\n\n##### into\n\n```css\ndiv {\n    color: red;\n    font-size: 12px;\n}\n```","ref":"readme.html#variables"},{"type":"extras","title":"Variables that create CSS variables on declaration - CSSEx","doc":"```css\n$*!primary red;\n\ndiv { color: <$primary$>; }\n```\n\n##### into\n\n```css\n:root {\n      --primary: red;\n}\n\ndiv { color: red; }\n```","ref":"readme.html#variables-that-create-css-variables-on-declaration"},{"type":"extras","title":"Scoped Variables by file and set only if undefined variables - CSSEx","doc":"##### file_1.cssex\n\n```css\n$!scope_variable_1 20px;\n$!scope_variable_2 blue;\n\ndiv { font-size: <$scope_variable_1$>; }\n\n@include file_2.cssex;\n\n#main {\n      font-size: <$scope_variable_1$>;\n      color: <$scope_variable_2$>;\n}\n```\n\n##### file_2.cssex\n\n```css\n$()scope_variable_1 16px;\n$!scope_variable_2 red;\n\n.something {\n      font-size: <$scope_variable_1$>;\n      color: <$scope_variable_2$>;\n}\n\n@include file_3.cssex;\n```\n\n##### file_3.cssex\n\n```css\n$?scope_variable_1 12px;\n$?scope_variable_2 green;\n\n.something-2 {\n      font-size: <$scope_variable_1$>;\n      color: <$scope_variable_2$>;\n}\n```\n\n##### into\n\n```css\ndiv { font-size: 20px; }\n#main {\n      font-size: 20px;\n      color: red;\n}\n\n.something {\n      font-size: 16px;\n      color: red;\n}\n\n.something-2 {\n      font-size: 20px;\n      color: red;\n}\n```\n\nVariables are literal values that can be used throughout a spreadsheet. They're inserted in place using the interpolation markers, `<$ variable_name $>`, or `$::variable_name`.\nThe declaration form is:\n\n`$!name_of_variable literal_value;`\n\n`literal_value` shouldn't be surround by quotes unless you want the quotes to be part of the value. It must always end with semi-colon followed by newline.\n\nUsing variables with `@include`'s directives allow one to share or create values that are overridable or specific to a stylesheet/include.\n\nWhen a stylesheet declares an `@include` the current variables in scope will be made available on the child.\nChild files (the one's `@include`d) can override the parent's variable value (after the child has been processed) if they declare them with `$!`,\n\nYou can otherwise limit this by declaring them with:\n`$()` which scopes the variable locally so they do not become available to the parent afterwards, and you can also conditionally set them with:\n\n`$?` which only sets them for the file if it isn't set in its scope yet. This allows you to create customisable themes, in that you can have a file(s) that specifies all colors, sizes and others, and then make the theme \"components\", set their needed values with `$?` which will populate those for the scope only if they haven't been declared previously, or override them in an included stylesheet with `$()`\n\nYou can also declare a variable with `$*!`, that does all the same but sets as well a CSS variable on the root element, with the same name as the variable declared, prefixed with `--`.\n\nIf you referr to a variable that hasn't been declared or not in scope you'll get an error.","ref":"readme.html#scoped-variables-by-file-and-set-only-if-undefined-variables"},{"type":"extras","title":"Assigns - CSSEx","doc":"Assigns are the equivalent of variables but for Elixir values and they have the same options and scoping as that of variables, but instead of being declared with `$!`, `$()` and `$?`, they're declared with `@!`, `@()` and `@?`.\n\nVariables are available inside EEx blocks or when calling functions. Since they can hold any elixir term they're great to create basic iterable structures to generate CSS based on repetition or iteration.\n\n```css\n@!colors %{\n\t primary: \"red\",\n\t secondary: \"rgb(120, 255, 80)\"\n};\n\n<%= for {color, val} <- @colors, reduce: \"\" do\n    acc ->\n    \tacc <> \"\"\"\n\n\t$*!#{color} #{val};\n\n\t.btn-#{color} {\n\t      \tbackground-color: #{val};\n\t}\n\n\t\"\"\"\nend %>\n\n```\n\ninto\n\n```css\n.root {\n      --primary: red;\n      --secondary: rgb(120, 255, 80)\n}\n\n.btn-primary { background-color: red }\n.btn-secondary { background-color: rgb(120, 255, 80) }\n```\n\n(while also creating both the `primary` and `secondary` variables that are accessible through the remaining spreadsheet and `@include`d children.\n\nInside EEx blocks they can be referred to with Elixir's `@assign_name` notation. When passing them to function as arguments they can be passed in as `@::assign_name`\n\nSo `@fn::name_of_function(@::colors)` would call that function with the argument being translated to the Elixir value of that variable.\nAs literal variables it will error if they're used without having been declared before or not available in scope.\n\nThe declaration form is:\n\n`@!name_of_variable {:tuple, %{elixir: \"map\"}};`\n\nIt has to be terminated with semicolon followed by newline.\n\n  \n\n### @expandable & @apply\n\nExpandables allow you to define utility classes that can be used inside any selector to add attributes & or selectors. It allows for then `@apply`ing those blocks in different ways. You can force the `@apply` to be exactly as it was resolved when declared and keep the selector hierarchies there defined in reference to the `@expandable` selector, you can use it to be dynamically evaluated, or to apply its hierarchies in the new context while using any variable interpolation as it occurred when defining. It might sound confusing but with an example is easier to see\n\n```scss\n$!color red;\n\n@expandable .hoverable {\n  color: <$color$>;\n  &:hover {\n    color: @fn::darken(<$color$>, 10);\n  }\n  container& {\n    background-color: black;\n  }\n}\n\n.class-1 {\n  @apply hoverable;\n}\n\n$!color blue;\n\n.class-2 {\n  @apply !hoverable;\n}\n\n.class-3 {\n  @apply ?hoverable;\n}\n\n.class-4 {\n  @apply hoverable;\n}\n```\n\n##### into\n\n```css\n.hoverable {\n  color:red\n}\n\n.hoverable:hover {\n  color:rgba(204,0,0,1.0)\n}\n\ncontainer .hoverable {\n  background-color:black\n}\n\n.class-1 {\n  color:red\n}\n\n.class-1:hover {\n  color:rgba(204,0,0,1.0)\n}\n\ncontainer .class-1 {\n  background-color:black\n}\n\n.class-2 {\n  color:red\n}\n\n.class-2 .hoverable:hover {\n  color:rgba(204,0,0,1.0)\n}\n\n.class-2 container .hoverable {\n  background-color:black\n}\n\n.class-3 {\n  color:blue\n}\n\n.class-3:hover {\n  color:rgba(0,0,204,1.0)\n}\n\ncontainer .class-3 {\n  background-color:black\n}\n\n.class-4 {\n  color:red\n}\n\n.class-4:hover {\n  color:rgba(204,0,0,1.0)\n}\n\ncontainer .class-4 {\n  background-color:black\n}\n```\n\nAs you can see, we had a variable `$!color` declared with the value `red`.\nThen we declared an `@expandable` block for the selector `.hoverable`, where we used interpolation for `color`, and had nesting selectors, one of which calling a function again with an interpolated value.\n\nThat is the top level declarations that came on top of the stylesheet:\n\n```css\n.hoverable {\n  color:red\n}\n\n.hoverable:hover {\n  color:rgba(204,0,0,1.0)\n}\n\ncontainer .hoverable {\n  background-color:black\n}\n```\nThen we declared a `.class-1` selector and applied the `hoverable` element we declared as `@expandable` before inside it (notice we don't use the `.` before its name).\n\nWhat this did was apply all the contents of that expandable element inside `.class-1`. You can see that the nesting `&` where applied to `.class-1`:\n\n```css\n.class-1 {\n  color:red\n}\n\n.class-1:hover {\n  color:rgba(204,0,0,1.0)\n}\n\ncontainer .class-1 {\n  background-color:black\n}\n```\n\nThen we overwrote the `color` variable with the value `blue`.\nOn `.class-2` we did the same as in `.class-1` but this time we prepended the name with `!`. This forces the expandable element to be inserted with the interpolation it had when evaluated originally and also with the nesting referring to the original `.hoverable` selector:\n\n```css\n.class-2 {\n  color:red\n}\n\n.class-2 .hoverable:hover {\n  color:rgba(204,0,0,1.0)\n}\n\n.class-2 container .hoverable {\n  background-color:black\n}\n```\n\nOn `.class-3` we did the same, but now prepended the expandable identifier with `?`. This made it be dynamically evaluated, both interpolation and nesting. You can see the color being blue and the result of `@fn::darken` being blue as well:\n\n```css\n.class-3 {\n  color:blue\n}\n\n.class-3:hover {\n  color:rgba(0,204,0,1.0)\n}\n\ncontainer .class-3 {\n  background-color:black\n}\n```\n\nLastly we did the same original non-prefix `@apply` in `.class-4` and in this case, the value of the variables used were the ones set at declaration time, `red`, but the nesting was still applied in terms of the current selector:\n\n```css\n.class-4 {\n  color:red\n}\n\n.class-4:hover {\n  color:rgba(204,0,0,1.0)\n}\n\ncontainer .class-4 {\n  background-color:black\n}\n```\n\nYou can also string together several elements to expand in a single `@apply`, with different evaluation scopes as well. The order of declaration is the order the attributes will be set.\n\n`@apply one-expandable ?another-expandable !a-different-one;`\n\n`@expandable` declarations specify a **single** class selector (e.g. `.class`) followed by a block `{ ... }`.\n\nThe `@expandable` directives  are placed in order of declaration at the top of the final stylesheet as individual selectors, but you can declare them from any file or part of the file as long as it's not inside a block, they must always be declared on a top level.\n\nThe only exception on their final placement is when using `@media` selectors inside `@expandable`. In those cases normal rules will go to the top but the media parts will be placed alongside the other media statements.\n\nThe `@apply` directive takes a list of white-space separated tokens, where each token refers to a previously declared `@expandable` block. You should not use the `.` of the original class to refer to them. You can additionally define their expansion mode with `?` (all elements dynamically evaluated) or `!` (as a static block as evaluated when declared). The default without prefix is a mix of both, variables with the values at the time of declaration and nesting dynamically evaluated inside the context of the block the `@apply` is declared in.\n\nKeep note that `<$ ... $>` interpolation is the only consistent form of interpolation. If you use, for instance in `@fn::...` call the form `$::color`, this will always be dynamically evaluated. EEx blocks follow the same pattern, if you want them to be expanded as they resolved at declaration you need to use the `!` prefix when `@apply`ing.\n\nNote that you can also declare `@media` attributes inside `@expandables` and nest them as well. Most times you'll want to use normal applying or `?`.\n\n\n </>","ref":"readme.html#assigns"},{"type":"extras","title":"Functions - CSSEx","doc":"```elixir\n\n@fn lighten_test(color, percentage) ->\n    {:ok, %CSSEx.HSLA{l: %CSSEx.Unit{value: l} = l_unit} = hsla} = \n                                                       CSSEx.HSLA.new_hsla(color)\n\n    {percentage, _} = Float.parse(percentage)\n\n    new_l = \n      case l + percentage do\n         n_l when n_l <= 100 and n_l >= 0 -> n_l\n\t n_l when n_l > 100 -> 100\n\t n_l when n_l < 0 -> 0\n      end\n\n    %CSSEx.HSLA{hsla | l: %CSSEx.Unit{l_unit | value: new_l}}\n    |> to_string\nend;\n\n$!red red;\n.test{color: @fn::lighten_test(<$red$>, 10)}\n.test{color: @fn::lighten_test(#fdf, 10);}\n```\n\n##### into\n\n```css\n.test {\n  color: hsla(0,100%,60%,1.0);\n  color: hsla(300,7%,15%,1.0);\n}\n```\n\nFunctions can execute any code, and can receive an arbitrary block of content, that is made available in its body as a variable named, `ctx_content`. They need to evaluate to either `{:ok, binary | iodata}` or `binary | iodata`. You do not have access to functions, assigns, or variables inside functions, but you can pass them as arguments. The returned binary is also parsed before being inserted so you can also  make use of them on the returned result.\nInside a function you can call any Elixir code.\n\nA function declaration has the following form:\n\n```scss\n@fn name_of_function(args, list) ->\n   body\nend;\n```\n\n`@fn` followed by the name and argument list followed by `->`. It must be terminated with `end`, semicolon followed by newline.\n\nAnd a function call for that function would be `@fn::name_of_function(red, #ffffff)`.\nEvery function can take a last optional parent that is made available in the body of the function as `ctx_content`. In this case the call would be:\n\n```scss\n@fn::name_of_function(red, #ffffff,\n   .content-made-available {\n     &p.inside-name-of-function { color: blue; }\n   }\n)\n```\n\nWhen successfully called the result of evaluating it replaces the function call in the stylesheet and is parsed as if it was normal cssex code. This means that if a function returns content that includes  function calls, or interpolation, or declare variables, all those things are evaluated once it returns.\n\n\n```scss\n@fn enforce_size(what, size) ->\n  \"#{what}: #{size};\" <>\n  \"min-#{what}: #{size};\" <>\n  \"max-#{what}: #{size};\"\nend;\n\n@fn enforce_wh(width, height) ->\n  \"\"\"\n  @fn::enforce_size(width, #{width})\n  @fn::enforce_size(height, #{height})\n  \"\"\"\nend;\n\n@fn enforce_square(size) ->\n  \"\"\"\n  @fn::enforce_size(width, #{size})\n  @fn::enforce_size(height, #{size})\n  \"\"\"\nend;\n\n@fn enforce_circle(size) ->\n  \"\"\"\n  @fn::enforce_square(#{size})\n  border-radius: 50%;\n  \"\"\"\nend;\n\n@fn my_squarer(element) ->\n  \"\"\"\n  #{element} {\n     #{ctx_content}\n     @fn::enforce_square(20px)\n  }\n  \"\"\"\nend;\n```\n\nAnd then using for instance:\n\n```scss\n.section {\n  @fn::my_squarer(&.inner, color: red;)\n}\n```\n\nResults in:\n\n```css\n.section.inner{\n  color:red;\n  width:20px;\n  min-width:20px;\n  max-width:20px;\n  height:20px;\n  min-height:20px;\n  max-height:20px;\n}\n```\n\nFunctions declared in a stylesheet (even `@included` children!) become available from then on. Further declarations of the same function name will override the previous one from then on.\n\n </>","ref":"readme.html#functions"},{"type":"extras","title":"Implemented Functions - CSSEx","doc":"There's 3 basic functions that ship with CSSEx and that you can use:\n\n- `@fn::lighten(color_to_lighten, percentage)`\nUse as `@fn::lighten(blue, 10)`\n\n- `@fn::darken(color_to_darken, percentage)`\nUse as `@fn::darken(blue, 10)`\n\n- `@fn::opacity(color, 0_to_1)`\nUse as `@fn::opacity(blue, 0.2)`\n\nNotice they don't take units, and opacity needs a well formed float, e.g. 0.4.\n\n\n </>","ref":"readme.html#implemented-functions"},{"type":"extras","title":"EEx blocks - CSSEx","doc":"```elixir\n@!breakpoints [\n  sm: {\"0px\", \"14px\"},\n  md: {\"768px\", \"16px\"},\n  lg: {\"992px\", \"18px\"},\n  xl: {\"1200px\", \"20px\"},\n  xxl: {\"1440px\", \"20px\"},\n];\n\n\n<%= for {breakpoint, {_size, val}} <- @breakpoints, reduce: \"\" do\n    acc ->\n    \tacc <> \"\"\"\n\t\n        .#{breakpoint}-section-title {\n\t   font-size: #{val};\n\t}\n\n\t\"\"\"\nend %>\n```\n\n##### into\n\n```css\n.sm-section-title { font-size: 14px }\n.md-section-title { font-size: 16px }\n.lg-section-title { font-size: 18px }\n// ... etc\n```\n\nAn EEx block has to evaluate to either a binary (a string) or an iodata list. It's declared as Elixir blocks but always with the opening tag including the equal sign: `<%=`.\nInside EEx blocks you can use assigns with `@name_of_assign` syntax.\n\nRight now you do not have access to either cssex variables or functions declared in the stylesheet but, again, the result of evaluating the block can contain any valid cssex construct which will be parsed afterwards in the context of the stylesheet as regular cssex before moving on to the remaining stylesheet.\n\n  \n\n### @at-rules (@media, @supports, @page, @font-face, @keyframes)\n\nFor `@media`, `@supports` and `@page` they can be declared inside blocks and can have declarations nested inside them as well. Notice that there's no semantic evaluation of the selectors defined on the `@at-rule` so if you nest them in a way that adds to an impossible to satisfy selector it will be placed as is.\n\n#### example\n\n```css\n.test {\n  @media screen and (max-width: 600px) {\n    div.example {\n      display: none;\n    }\n\n    font-family: Arial;\n  }\n                   \t\n  color: red;\n}\n\n@media screen and (max-width: 600px) {\n  .test { background-color: black; }\n}\n```\n\n##### into\n\n```css\n.test {\n  color:red\n}\n@media screen and (max-width:600px) {\n  .test div.example {\n    display:none\n  }\n  .test {\n    background-color:black;\n    font-family:Arial\n  }\n}\n```\n\nNesting requires you to only declare the part that is to be added to the parent declarator:\n\n```css\n@media screen {\n  @media screen and (max-width: 600px) {\n     div {\n       color: red;\n     }\n  }\n}\n```\n\n##### into\n\n```css\n@media screen screen and (max-width:600px) {\n  div{\n    color:red\n  }\n}\n```\nWhich is wrong of course. The nested media should have been declared with `@media and (max-width: 600px)`.\n\nThese same rules apply to `@supports` and `@page`.\n\nFor `@keyframes` the identifier after the `@keyframes` is used to identify it so any declarations for the same identifier will be coalesced into the same one.\n\n`@font-face` declarations each creates its own one in the final stylesheet.\n\n\n </>","ref":"readme.html#eex-blocks"},{"type":"extras","title":"Comments - CSSEx","doc":"```css\n// you can use comments in both of these forms, all text will be stripped out\n/* some\n\nother {\n   .commented-out part {\n\n}\n*/\n```","ref":"readme.html#comments"},{"type":"extras","title":"Reserved Tokens - CSSEx","doc":"Using any of these outside of the syntax shown before can lead to errors:\n\n```\n&\n@!\n@?\n@()\n@::\n$!\n$*!\n$?\n$*?\n$()\n$*()\n$::\n@include\n@expandable\n@apply\n@fn\n<% ....\n<$ ....\n//\n/* ....\n```","ref":"readme.html#reserved-tokens"},{"type":"extras","title":"Caveats - CSSEx","doc":"CSSEx does merging of selector declarations when they've been exactly specified (or resolved to) as the same.\n\nFor instance\n\n```css\ndiv { color: green; }\n.sample { color: red; }\n@include ../shared/samples.cssex;\n```\n\nAnd `../shared/samples.cssex`:\n\n```css\n$()tag div\n<$tag$> { background-color: white; color: orange; }\n```\n\nThe final output will always have `div` declared first, and duplicate attributes will hold the value according to their declaration order. Since the last evaluated `div` selector set the color to `orange` that's the color that the final stylesheet will hold:\n\n```css\ndiv {background-color: white, color: orange }\n.sample { color: red }\n```\n\nInstead of\n\n```css\ndiv { color: green; }\n.sample { color: red; }\ndiv { background-color: white, color: orange }\n```\n\nThis means it can condense the final output, and also that it's ok to `@apply` multiple `@expandable` classes to an element as any repeated attributes are just replaced, or define selectors with repeated attributes in different places as they coalesce into a single representations.\n\nIf you want multiple declarations to not be merged right now the only possibility is to have different entry points that produce plain css files and use them with regular `@import` rules on a final css. \n\nThe final output always follows this format:\n\n```\n@charset\n@imports\n@font-face\n**:root { variables }**\n@expandables\n**all regular selectors and their rules**\n@media\n@keyframes\n\\n\n```","ref":"readme.html#caveats"},{"type":"extras","title":"Installation - CSSEx","doc":"This is an early release of this library, it's available in [hex.pm](https://hex.pm).\nIts initial release in a smaller version than 1.0 is just meant to signal that there might be some little details (line/col information on errors, etc) that might not be 100% accurate although there aren't intended breaking changes in the roadmap.\n\nIt's fully functioning, has a relevant test suite and has been used to generate a smallish CSS framework that you can see in [simples](lib/test/files/includes/simples/simples.cssex) without problems. It has been tested as a standalone, in umbrellas and with webpack, on Linux Ubuntu 20.04 LTS, using `inotify-tools` as the file watcher, it should work with any other setup.\n\nThe only dependency is on `file_system` for the file watcher.\nIt's been used while developing and the tasks have been tested for deployment on [render.com](https://render.com) as part of the build script when assembling a release.\n\nTo install from `hex.pm` use:\n\n```elixir\n\ndefp deps do\n     [\n        {:cssex, \"~> 1.0\"}\n     ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - CSSEx","doc":"If you want to use other folder than priv you need to use an expandable path, e.g. the phoenix assets folder, you can do it by using, for instance in this case for an umbrella with a phoenix app:\n\n```elixir\n\nconfig :yourapp_web, CSSEx,\n  entry_points: [\n    {\n      \"../../../../apps/yourapp_web/assets/cssex/app.cssex\",","ref":"readme.html#usage"},{"type":"extras","title":"entry - CSSEx","doc":"\"../../../../apps/yourapp_web/assets/css/app.css\"","ref":"readme.html#entry"},{"type":"extras","title":"output - CSSEx","doc":"}\n  ]\n```\n\nYou can also configure CSSEx to pretty print the css, with `pretty_print?: true` on the config.\n\nThis is usually the case if you want to integrate with Webpack, or esbuild, this will output the resulting css file and webpack/esbuild can just then use `app.css` as regularly.\nYou can specify as many entry points as wanted.\n\nIf you are using the priv folder directly you can simply use it as:\n\n```elixir\n\nconfig :yourapp_web, CSSEx,\n  entry_points: [{\"priv/static/cssex/base.cssex\", \"priv/static/css/base.css\"}]\n```\n\n\n#### Adding the file watcher to your application supervision tree.\n\nFinally add to your `yourapp_web` application file (marked with `# add this`):\n\n```elixir\ndefmodule YourAppWeb.Application do\n   @moduledoc false\n   @env Mix.env() # add this\n\n   use Application\n\n   def start(_type, _args) do\n    children = [\n      YourAppWeb.Telemetry,\n      YourAppWeb.Endpoint\n    ] |> maybe_add_cssex() # add this\n\n    opts = [strategy: :one_for_one, name: YourAppWeb.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  # ... other code\n\n  # add this\n  def maybe_add_cssex(children) do\n    case @env do\n      :prod ->\n        children\n\n      _ ->\n        children ++\n          [\n            Supervisor.child_spec(%{id: CSSEx, start: {CSSEx, :start_link, [cssex_config()]}},\n              type: :worker\n            )\n          ]\n    end\n  end\n\n  def cssex_config() do\n    Application.get_env(:yourapp_web, CSSEx)\n    |> CSSEx.make_config(Application.app_dir(:yourapp_web))\n  end\nend\n```\n\nThis will define an entry point file of `priv/static/cssex/base.cssex` which will output its parsed content into `priv/static/css/base.css`.\nIn this case you'll need to create the folder and cssex file as well. It will log errors if the file doesn't exist.\n\nNow whenever you change something in the entry point or any of the files it depends on (`@include`) it will automatically recompile.\n\nThe reason we use `@env Mix.env()` is so that we don't actually add CSSEx as an application to start when running in production.\n\n#### Production assets\nIf you wish to process and generate the files during assembling of a release add a simple mix task to your build script:\n\n`mix cssex.parser --c yourapp_web`\n\nPlace this before the `npm run deploy` or any script you use to bundle your js/css and/or before `mix phx.digest`.\n\n\nAnd this will use the same environment.","ref":"readme.html#output"},{"type":"extras","title":"Tasks - CSSEx","doc":"Besides the automatic file watcher for development purposes it's also included a task for processing cssex files into css files.\n\nYou can read [cssex.parser task](https://github.com/mnussbaumer/cssex/lib/mix/tasks/css.parser.ex) for other details.\nThe base syntax is:\n\n```\n# if using entry points defined in a config similar to the watcher\nmix cssex.parser --c yourapp_web \n\n# if using manual entry points\nmix cssex.parser --e path/to/file.cssex=path/to/output.css\n```","ref":"readme.html#tasks"},{"type":"extras","title":"Dynamic/RunTime parsing - CSSEx","doc":"You can parse a file or a string of CSS syntax by using directly the functions in CSSEx.Parser. For instance, to parse a file you can use:\n\n`CSSEx.Parser.parse_file(\"/base/path\", \"/base/path/file.css\")`\nYou need to pass both the folder path and the file path independently.\n\nYou can also pass `pretty_print?` option by using `CSSEx.Parser.parse_file(\"/base/path\", \"/base/path/file.css\", pretty_print?: true)`\n\nIf you want to write directly to a file call it instead as:\n`CSSEx.Parser.parse_file(\"/base/path\", \"/base/path/file.css\", \"/output/path.css\", pretty_print?: true)`\n\nYou can do the same to parse directly a string containing CSS syntax:\n`CSSEx.Parser.parse(\"div{color:white;}\", pretty_print?: true)`\n\nAgain, refer to the parser file to see the syntax details.","ref":"readme.html#dynamic-runtime-parsing"},{"type":"extras","title":"Internals - CSSEx","doc":"Right now the library is mostly for use as regular pre-processor with a syntax that aids in building re-usable themes but it's been designed up 'till now to be expandable and to accomodate more use cases, such as dynamically feeding assigns, variables, functions, to templates, producing tables of css selectors and other details for post-processing, and generating on the fly CSS.\n\nNone of this is yet implemented in the form of a consistent api nor totally defined, but if you have a use case for any of those open an issue to discuss.\n\nOther than that check the parser.ex to see how you can start an individual parser that can write to a file or return a binary with the result of the parsing.","ref":"readme.html#internals"},{"type":"extras","title":"About - CSSEx","doc":"![Cocktail Logo](https://github.com/mnussbaumer/cssex/blob/master/logo/cocktail_logo.png?raw=true \"Cocktail Logo\")\n\n[© rooster image in the cocktail logo](https://commons.wikimedia.org/wiki/User:LadyofHats)","ref":"readme.html#about"},{"type":"extras","title":"Copyright - CSSEx","doc":"```\nCopyright [2021-∞] [Micael Nussbaumer]\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this \nsoftware and associated documentation files (the \"Software\"), to deal in the Software\nwithout restriction, including without limitation the rights to use, copy, modify, \nmerge, publish, distribute, sublicense, and/or sell copies of the Software, and to \npermit persons to whom the Software is furnished to do so, subject to the following \nconditions:\n\nThe above copyright notice and this permission notice shall be included in all copies\nor substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,\nINCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A\nPARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT\nHOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE\nSOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n```","ref":"readme.html#copyright"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}